/*
 * ICFP 2002 Programming Contest
 * Entry by Seth Bruder
 *
 */

package sethbruder.icfp2002.entry;

import java.util.HashSet;
import java.util.Set;

public final class Board {
	public static final byte EMPTY = 0;
	public static final byte WATER = 1;
	public static final byte WALL = 2;
	public static final byte BASE = 3;

	private static final char[] typeChars = {'.', '~', '#', '@'};

	public final byte[][] board;
	private final int maxDist;

	public Board(byte[][] board) {
		if (board.length == 0) 
			throw new RuntimeException("Illegal board size.");
		this.board = board;
		this.maxDist = board.length * board[0].length + 1;
	}

	public String toString(int myX, int myY, Set otherPlayerPointSet) {
		StringBuffer sb = new StringBuffer();

		if (board.length == 0) return "";
		for (int y = (board[0].length - 1); y >= 0; y--) {
			for (int x = 0; x < board.length; x++) {
				if ((x == myX) && (y == myY)) sb.append('*');
				else if (otherPlayerPointSet.contains(new Board.Point(x, y)))
					sb.append('x');
				else sb.append(typeChars[board[x][y]]);
			}
			sb.append('\n');
		}
		return sb.toString();
	}

	public String toString() {
		return toString(-1, -1, new HashSet());
	}

	public int getMaxDist() {
		return maxDist;
	}

	public static final class Point {
		public final int x;
		public final int y;

		public Point(int x, int y) {
			this.x = x;
			this.y = y;
		}

		public boolean equals(Object o) {
			if (!(o instanceof Point)) return false;
			Point p = (Point)o;
			return ((p.x == this.x) && (p.y == this.y));
		}

		public int hashCode() { return (x + y); }
	}
}
