/*
 * ICFP 2002 Programming Contest
 * Entry by Seth Bruder
 *
 */

package sethbruder.icfp2002.entry;

public final class Move {

	// move types

	public static final int NULL_MOVE_TYPE = 0;
	public static final int NORTH = 1;
	public static final int SOUTH = 2;
	public static final int EAST = 3;
	public static final int WEST = 4;
	public static final int PICK = 5;
	public static final int DROP = 6;
	public static final int POSITION = 7;

	private static final String[] typeNames
		= {"None", "Move N", "Move S", "Move E", "Move W", "Pick", "Drop",
		"Position"};

	public static final Move NORTH_MOVE = new Move(NORTH);
	public static final Move SOUTH_MOVE = new Move(SOUTH);
	public static final Move EAST_MOVE = new Move(EAST);
	public static final Move WEST_MOVE = new Move(WEST);

	public final int bid;
	public final int type;
	public final int[] ids;
	public final int x, y;

	private Move(int type) {
		this(1, type, new int[0]);
	}

	public Move(int type, int x, int y) {
		if (type != POSITION) throw new RuntimeException();
		this.bid = 0;
		this.type = type;
		this.ids = null;
		this.x = x;
		this.y = y;
	}

	public Move(int type, int id) {
		this.bid = 0;
		this.type = type;
		this.ids = new int[] {id};
		this.x = this.y = -1;
	}

	public Move(int bid, int type, int[] ids) {

		if (bid == 0) throw new RuntimeException("Attempt to bid nothing.");
		if (ids == null) ids = new int[0];
		
		this.bid = bid;
		this.type = type;
		this.ids = ids;
		this.x = this.y = -1;
	}

	public String toString() {
		if (type == POSITION) return ("X " + x + " Y " + y);

		StringBuffer sb = new StringBuffer();

		if (bid != 0) sb.append(bid + " ");
		sb.append(typeNames[type]);
		for (int i = 0; i < ids.length; i++)
			sb.append(" " + ids[i]);
		return sb.toString();
	}
}
